// bird's eye view js functions
// depends on plotly.js

function meshCone(topR,lowR,topZ,lowZ,faces) {
    x=[]
    y=[]
    z=[]
    for (i=0;i<faces;i++) {
        x[i*2]=topR*Math.cos(i*Math.PI*2.0/faces)
        y[i*2]=topR*Math.sin(i*Math.PI*2.0/faces)
        z[i*2]=topZ
        x[i*2+1]=lowR*Math.cos(i*Math.PI*2.0/faces)
        y[i*2+1]=lowR*Math.sin(i*Math.PI*2.0/faces)
        z[i*2+1]=lowZ
    }
    cone={
        name: "Abstraction Cone"
        ,x: x
        ,y: y
        ,z: z
        ,text: "Abstraction Cone"
        ,type: "mesh3d"
        ,showscale: false
        ,opacity: 0.3
        ,hoverinfo: 'skip'
        ,showlegend: true
    }
    return cone
}

function timeArrow(angle,faces) {
    topR=1.1
    lowR=1.05
    deltaR=(topR-lowR)/faces
    topZ=0
    lowZ=0
    x=[]
    y=[]
    z=[]
    for (i=0;i<faces;i++) {
        radAngle=Math.PI/2-i*Math.PI*2.0*angle/(360*faces)
        x[i*2]=topR*Math.cos(radAngle)
        y[i*2]=topR*Math.sin(radAngle)
        z[i*2]=topZ
        x[i*2+1]=lowR*Math.cos(radAngle)
        y[i*2+1]=lowR*Math.sin(radAngle)
        z[i*2+1]=lowZ
        topR-=deltaR
    }
    cone={
        name: "Time Arrow"
        ,x: x
        ,y: y
        ,z: z
        ,text: "Start Here"
        ,type: "mesh3d"
        ,showscale: false
        ,opacity: 0.8
        ,color: 'yellow'
        ,hoverinfo: 'skip'
        ,showlegend: false
    }
    return cone
}

function abstractionCone(lowZ,topR=0.5,lowestR=1.0) {
    lowR=(lowestR)*(1.0-lowZ)/1.0

    return meshCone(topR,lowR,1.0,lowZ,30)
}

function drawDoc(doc) {
    var wordTrace = {
        name: "Original Text"
        ,x: doc.words.x,y: doc.words.y,z:doc.words.z
        ,text: doc.words.label
        ,mode: 'text'
        ,textposition: 'center'
        ,textfont: {
            color: 'blue'
            ,size: 7
        }
        ,customdata: doc.words.context
        ,hovertemplate: '%{customdata}'
        ,type: 'scatter3d'
    };
    var lemmaTrace = {
        name: "Lemmas"
        ,x: doc.lemmas.x,y: doc.lemmas.y,z:doc.lemmas.z
        ,text: doc.lemmas.label
        ,mode: 'text'
        ,textposition: 'center'
        ,textfont: {
            color: 'green'
            ,size: doc.lemmas.size
        }
        ,customdata: doc.lemmas.context
        ,hovertemplate: ' %{z}<br>%{customdata}'
        ,type: 'scatter3d'
    };
    var stopTrace = {
        name: "Stop Words"
        ,x: doc.stop.x,y: doc.stop.y,z:doc.stop.z
        ,text: doc.stop.label
        ,mode: 'text'
        ,textposition: 'center'
        ,textfont: {
            color: 'black'
            ,size: doc.stop.size
        }
        ,customdata: doc.stop.context
        ,hovertemplate: ' %{z}<br>%{customdata}'
        ,type: 'scatter3d'
    }

    var absCone = abstractionCone(0.15)
    var arrow = timeArrow(15,10)
    var data=[wordTrace,lemmaTrace,stopTrace,absCone,arrow]
    //var data=[absCone]
    layout = {
        margin: { t:12}
        ,"title":{
            "text":"Glorified Statistics"
            , "x": 0.1
            , "xanchor": "left"
            , "y": 0.98
            ,"yanchor": "top"
        }
        ,"scene": {
            "camera": {
            "eye": {x: -1.0, y: -1.2, z: 1.0}
            }
        }
    }
    console.log("yes, I am here")
    view = document.getElementById('bevDiv');
    Plotly.newPlot( view, data,layout )
        //{margin: { t: 0 } })

}

function drawFromURL() {
    const urlParams = new URLSearchParams(location.search);
    let jdoc=null
    for (const [key,value] of urlParams) { 
        console.log(`${key}:${value}`);
        if (key=="document") {
            jdoc=value
        }
    }
    if (jdoc!=null) {
        fetch(jdoc)
            .then(response => response.json())
            .then(json => drawDoc(json));
    }
}